from django.db import models
from django.utils import timezone
from django.conf import settings 
import time

User = settings.AUTH_USER_MODEL
    
# references
class Package(models.Model):
    name = models.CharField(max_length=50)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Packages"
        
    def __str__(self):
        return self.name
    
    
    
class ExpenseType(models.Model):
    name = models.CharField(max_length=50)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Expense Types"
    def __str__(self):
        return self.name
    
    
    
class Bank(models.Model):
    name = models.CharField(max_length=50)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Banks"
    
    def __str__(self):
        return self.name
    
    
    
class PaymentMode(models.Model):
    name = models.CharField(max_length=50)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Payment Modes"
        
    def __str__(self):
        return self.name
    
    
class BankTransactionType(models.Model):
    name = models.CharField(max_length=50)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    class Meta:
        verbose_name_plural = "Bank Transaction Types"
    
    def __str__(self):
        return self.name
    
    
# Admintration
class CompanyProfile(models.Model):
    name = models.CharField(max_length=50, default=None)
    email = models.EmailField(max_length=50, null=True, blank=True)
    phone = models.CharField(max_length=50, null=True, blank=True)
    address = models.CharField(max_length=256, null=True, blank=True)
    website = models.CharField(max_length=50, null=True, blank=True)
    vatNo = models.CharField(max_length=50, null=True, blank=True)
    vatRate = models.DecimalField(max_digits=10, decimal_places = 2, null=True, blank=True)
    NHIL_Rate = models.DecimalField(max_digits=10, decimal_places = 2, null=True, blank=True)
    A4_Printer = models.BooleanField(default=False)
    no_of_receipts = models.IntegerField(default=0)
    show_vat = models.BooleanField(default=False)
    firstTextBelowReceipt = models.CharField(max_length=50, null=True, blank=True)
    secondTextBelowReceipt = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    class Meta:
        verbose_name_plural = "Company Profiles"
        
    def __str__(self):
        return self.name
    
    
    


class CompanyLocation(models.Model):
    name = models.CharField(max_length=50)
    phone = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Company Locations"
        
    def __str__(self):
        return self.name
    
    

class ProductCategory(models.Model):
    name = models.CharField(max_length=50)
    description = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Product Categories"
        
    def __str__(self):
        return self.name


class Products(models.Model):
    name = models.CharField(max_length=50)
    baseUnit = models.ForeignKey(Package, on_delete=models.PROTECT, null=True)
    category = models.ForeignKey(ProductCategory, on_delete=models.PROTECT, null=True, blank=True)
    reorderLevel = models.IntegerField(default=0)
    is_active = models.BooleanField(default=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Products"
        
    def __str__(self):
        return self.name 



class ProductPrice(models.Model):
    sale_type_choices = (('retail', 'retail'), ('wholesale', 'wholesale'))
    product = models.ForeignKey(Products, on_delete=models.PROTECT, null=True, blank=True)
    package = models.ForeignKey(Package, on_delete=models.PROTECT, null=True, blank=True)
    qty_in_package = models.IntegerField(default=1)
    stockLevel = models.IntegerField(default=0)
    # branch = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True)
    cost_price = models.DecimalField(default=0.0, max_digits=10, decimal_places=2)
    # PCP = models.DecimalField(default=0.0, null=True, blank=True, max_digits=10, decimal_places=2)
    VAT = models.DecimalField(default=0.0, max_digits=10, decimal_places=2)
    NHIL = models.DecimalField(default=0.0, max_digits=10, decimal_places=2)
    margin = models.DecimalField(default=0.0, max_digits=10, decimal_places=2)
    selling_price = models.DecimalField(default=0, max_digits=10, decimal_places=2)
    companylocation = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True)
    sale_type = models.CharField(max_length=50, null=True, blank=True, choices=sale_type_choices)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    @property
    def get_VAT(self):
        if self.cost_price and self.VAT:
            percentage = (self.VAT/100) * self.cost_price
            percentage = format(percentage, '.2f') 
            return percentage
        else:
            return 0
        
    @property
    def get_NHIL(self):
        if self.cost_price and self.NHIL:
            nhil = (self.cost_price/100) * self.NHIL
            nhil = format(nhil, '.2f')
            return nhil
        else:
            return 0
        
        
    @property
    def get_selling_price(self):
        if self.cost_price and self.margin:
            vat = self.get_VAT
            nhil = self.get_NHIL
            selling_price = float(vat) + float(nhil) + float(self.cost_price) + float(self.margin)
            selling_price = format(selling_price, '.2f')
            return selling_price
        else:
            vat = self.get_VAT
            nhil = self.get_NHIL
            selling_price = float(vat) + float(nhil) + float(self.cost_price)
            selling_price = format(selling_price, '.2f')
            return selling_price
        
        
    class Meta:
        verbose_name_plural = "Product Prices"
    
    def save(self, *args, **kwargs):
        
        self.selling_price = self.get_selling_price
        super(ProductPrice, self).save(*args, **kwargs)
         
    def __str__(self):
        return self.product.name
    
    

# class ProductStock(models.Model):
#     product = models.ForeignKey(Products, on_delete=models.PROTECT, null=True)
#     package = models.ForeignKey(Package, on_delete=models.PROTECT, null=True)
#     price = models.DecimalField(max_digits=10, decimal_places=2)
#     initialStock = models.IntegerField(default=0, null=True, blank=True)
#     stockLevel = models.IntegerField(default=0, null=True, blank=True)
#     created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
#     edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
#     date_created = models.DateTimeField(default=timezone.now)
#     date_updated = models.DateTimeField(auto_now=timezone.now)
    
#     def save(self, *args, **kwargs):
#         user = kwargs.pop('user', None)
#         if user:
#             if self.pk:
#                 self.edited_by = user
#             else:
#                 self.created_by = user
#         super(ProductPrice, self).save(*args, **kwargs)
        
#     def __str__(self):
#         return self.product.name



class Customer(models.Model):
    name = models.CharField(max_length=50)
    phone = models.CharField(max_length=50, null=True, blank=True)
    email = models.EmailField(max_length=50, null=True, blank=True)
    address = models.CharField(max_length=50, null=True, blank=True)
    debit = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    credit = models.DecimalField(max_digits=10, decimal_places = 2, default=0)
    creditLimit = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_credit = models.DecimalField(max_digits=10, decimal_places=2, default=0, blank=True)
    total_paid = models.DecimalField(max_digits=10, decimal_places=2, default=0, blank=True)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    opening_balance = models.DecimalField(max_digits=10, decimal_places=2, default=0, )
    is_active = models.BooleanField(default=True)
    registration_date = models.DateField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
     
    class Meta:
        verbose_name_plural = "Customers"
    
    # @property
    # def get_balance(self):
    #     balance = (self.total_credit) + (self.total_paid)
    #     balance = format(balance, '.2f')
    #     return balance
    
    # def save(self, *args, **kwargs):
    #     self.balance = self.get_balance
    #     super(Customer, self).save(*args, **kwargs)   
    
    
        
    def __str__(self):
        return self.name
    
    
    
class Supplier(models.Model):
    name = models.CharField(max_length=50)
    phone = models.CharField(max_length=50, null=True, blank=True)
    email = models.EmailField(max_length=50, null=True, blank=True)
    address = models.CharField(max_length=50, null=True, blank=True)
    debit = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    credit = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    is_active = models.BooleanField(default=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Suppliers"
        
        
    def __str__(self):
        return self.name
    
    
    
    
class BankAccounts(models.Model):
    account_type_choices = (('savings', 'savings'), ('current', 'current'))
    
    account_number = models.CharField(max_length=50, null=True, blank=True)
    account_name = models.CharField(max_length=50, null=True, blank=True)
    opening_date = models.DateField(null=True, blank=True)
    bank = models.ForeignKey(Bank, on_delete=models.PROTECT, null=True)
    account_type = models.CharField(max_length=50, null=True, blank=True, choices=account_type_choices)
    opening_balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    current_balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Bank Accounts"
        
    def __str__(self):
        return str(self.account_number)
    
# 00552196612

class CreditSalePayment(models.Model):
    payment_date = models.DateTimeField()
    customer = models.ForeignKey(Customer, on_delete=models.PROTECT, null=True)
    company_location = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    paid_by = models.CharField(max_length=50, default=None)
    payment_mode = models.ForeignKey(PaymentMode, on_delete=models.PROTECT, null=True)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.ForeignKey(Bank, on_delete=models.PROTECT, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    class Meta:
        verbose_name_plural = "Credit Sale Payments"
        
        
    def __str__(self):
        return f'{self.customer} {self.amount} {self.payment_date}'



class StockPurchasePayment(models.Model):
    payment_date = models.DateTimeField(default=timezone.now)
    supplier = models.ForeignKey(Supplier, on_delete=models.PROTECT, null=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    paid_by = models.CharField(max_length=50, default=None)
    payment_mode = models.ForeignKey(PaymentMode, on_delete=models.PROTECT, null=True)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.ForeignKey(Bank, on_delete=models.PROTECT, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='modified_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    class Meta:
        verbose_name_plural = "Stock Purchase Payments"
    
    def __str__(self):
        return f'{self.supplier} {self.amount} {self.payment_date}'




class Sales(models.Model):
    sales_date = models.DateTimeField(default=timezone.now)
    sales_id = models.CharField(max_length=50, null=True, blank=True, default='PD-123')
    customer = models.CharField(max_length=64, blank=True, null=True)
    customer_phone = models.CharField(max_length=64, blank=True, null=True)
    credit_customer = models.ForeignKey(Customer, on_delete=models.PROTECT, null=True, blank=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2)
    selling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_discount = models.DecimalField(max_digits=10, decimal_places=2)
    company_location = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.CharField(max_length=64, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    momo = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cash = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    visa = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    payee_name = models.CharField(max_length=64, null=True, blank=True)
    discount_authorizer = models.CharField(max_length=64, null=True, blank=True)
    sale_type = models.CharField(max_length=64, null=True, blank=True)
    status = models.BooleanField(default=True)
    status_approved_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='status')
    stock_approval_date = models.DateTimeField(null=True, blank=True)
    
    # cash segmentation
    cedi1 = models.IntegerField(default=0)
    cedi2 = models.IntegerField(default=0)
    cedi5 = models.IntegerField(default=0)
    cedi10 = models.IntegerField(default=0)
    cedi20 = models.IntegerField(default=0)
    cedi50 = models.IntegerField(default=0)
    cedi100 = models.IntegerField(default=0)
    cedi200 = models.IntegerField(default=0)
    gp10 = models.IntegerField(default=0)
    gp20 = models.IntegerField(default=0)
    gp50 = models.IntegerField(default=0)
    
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Sales"
        
          
    def __str__(self):
        return f'{self.customer} {self.net_amount} {self.sales_id}'
    
    



class stockTransfer(models.Model):
    stock_transfer_date = models.DateTimeField(default=timezone.now)
    stock_transfer_id = models.CharField(max_length=50, null=True, blank=True, default='STF-123')
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    from_company = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True, related_name='from_%(class)s_set')
    to_company = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True, related_name='to_%(class)s_set')
    sales_date = models.DateTimeField(default=timezone.now)
    customer = models.CharField(max_length=64, blank=True, null=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    selling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_discount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.CharField(max_length=64, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    momo = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cash = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    visa = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    payee_name = models.CharField(max_length=64, null=True, blank=True)
    product = models.CharField(max_length=64, null=True, blank=True)
    package = models.CharField(max_length=64, null=True, blank=True)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    qty = models.CharField(max_length=64, null=True, blank=True)
    discount_authorizer = models.CharField(max_length=64, null=True, blank=True)
    productprice_id = models.IntegerField(null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)   
        
    class Meta:
        verbose_name_plural = "Stock Transfers"
        
        
    def __str__(self):
        return f'{self.customer} {self.net_amount} {self.stock_transfer_id}'
    
    


class stockPurchase(models.Model):
    stock_purchase_date = models.DateTimeField(default=timezone.now)
    stock_purchase_id = models.CharField(max_length=50, null=True, blank=True, default='STF-123')
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    sales_date = models.DateTimeField(default=timezone.now)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_discount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.CharField(max_length=50, null=True, blank=True)
    bank = models.CharField(max_length=64, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    momo = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cash = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    visa = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    payee_name = models.CharField(max_length=64, null=True, blank=True)
    product = models.CharField(max_length=64, null=True, blank=True)
    package = models.CharField(max_length=64, null=True, blank=True)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    qty = models.CharField(max_length=64, null=True, blank=True)
    supplier = models.ForeignKey(Supplier, on_delete=models.PROTECT, null=True, blank=True)
    CompanyLocation = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True)
    discount_authorizer = models.CharField(max_length=64, null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)   
    
        
    class Meta:
        verbose_name_plural = "Stock Transfers"
        
        
    def __str__(self):
        return f'{self.supplier} {self.net_amount} {self.stock_purchase_id}'
        
        

class StockPurchaseItem(models.Model):
    sales = models.ForeignKey(stockPurchase, on_delete=models.CASCADE)
    product = models.CharField(max_length=50)
    package = models.CharField(max_length=50)
    total = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    new_selling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2)
    new_stock = models.CharField(max_length=64, null=True, blank=True)
    qty = models.CharField(max_length=50, null=True, blank=True)
    unit_discount = models.DecimalField(max_digits=10, decimal_places=2)
    productprice_id = models.IntegerField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)    
    
    
    class Meta:
        verbose_name_plural = "Sale Items"
        
        
    def __str__(self):
        return f'{self.product} {self.total}'


class Colors(models.Model):
    color = models.CharField(max_length=25)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)   
    
    def __str__(self):
        return self.color
    
          
class BatchPaleting(models.Model):
    product = models.ForeignKey(ProductPrice, on_delete=models.PROTECT)
    batch_number = models.CharField(max_length=50)
    active = models.BooleanField(default=True)
    color = models.ForeignKey(Colors, on_delete=models.PROTECT)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)  
    
    
    def __str__(self):
        return self.batch_number
    

class SalesItem(models.Model):
    sales = models.ForeignKey(Sales, on_delete=models.CASCADE)
    product = models.CharField(max_length=50)
    package = models.CharField(max_length=50)
    total = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2)
    qty = models.CharField(max_length=50, null=True, blank=True)
    unit_discount = models.DecimalField(max_digits=10, decimal_places=2)
    productprice_id = models.IntegerField(null=True, blank=True)
    batch_number1 = models.ForeignKey(BatchPaleting, on_delete=models.PROTECT, null=True, blank=True, related_name="Batch_number1")
    batch_number2 = models.ForeignKey(BatchPaleting, on_delete=models.PROTECT, null=True, blank=True, related_name="Batch_number2")
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)    
    
    
    class Meta:
        verbose_name_plural = "Sale Items"
        
    
        
    def __str__(self):
        return f'{self.product} {self.total - (self.unit_discount*int(self.qty)) }'
    
    
    
    

class Protocol(models.Model):
    sales_date = models.DateTimeField(default=timezone.now)
    sales_id = models.CharField(max_length=50, null=True, blank=True, default='PC-123')
    customer = models.CharField(max_length=64, blank=True, null=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2)
    company_location = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    status = models.BooleanField(default=False)
    status_approved_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='protocol')
    stock_approval_date = models.DateTimeField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Protocols"
        
          
    def __str__(self):
        return f'{self.customer} {self.net_amount} {self.sales_id}'
    



class ProtocolSaleItem(models.Model):
    sales = models.ForeignKey(Protocol, on_delete=models.CASCADE)
    product = models.CharField(max_length=50)
    package = models.CharField(max_length=50)
    total = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2)
    qty = models.CharField(max_length=50, null=True, blank=True)
    productprice_id = models.IntegerField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    
    class Meta:
        verbose_name_plural = "Protocol Sale Items"
    
    
    
    def __str__(self):
        return f'{self.product} {self.total}'




class Expenses(models.Model):
    companylocation = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True)
    expense_date = models.DateTimeField(default=timezone.now)
    expense_type = models.ForeignKey(ExpenseType, on_delete=models.PROTECT, null=True, blank=True)
    description = models.CharField(max_length=125, null=True, blank=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    payment_mode = models.ForeignKey(PaymentMode, on_delete=models.PROTECT, null=True, blank=True)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateField(null=True, blank=True)
    bank_account = models.ForeignKey(BankAccounts, on_delete=models.PROTECT, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='edited_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)

    class Meta:
        verbose_name_plural = "Expenses"
        
        
    def __str__(self):
        return str(self.expense_type) + ' ' + str(self.amount)
    
    
    
class BankTransaction(models.Model):
    companylocation = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True, blank=True)
    transaction_date = models.DateTimeField(default=timezone.now)
    transaction_type = models.ForeignKey(BankTransactionType, on_delete=models.PROTECT, null=True, blank=True)
    notes = models.CharField(max_length=125, null=True, blank=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    transacted_by = models.CharField(max_length=64, null=True, blank=True)
    payment_mode = models.ForeignKey(PaymentMode, on_delete=models.PROTECT, null=True, blank=True)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateField(null=True, blank=True)
    bank_account = models.ForeignKey(BankAccounts, on_delete=models.PROTECT, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    edited_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='edited_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    
    
    class Meta:
        verbose_name_plural = "Bank Transactions"
    
    def __str__(self):
        return str(self.transaction_type) + ' ' + str(self.amount)
    
    
    
    
class Stock(models.Model):
    trans_date = models.DateTimeField(default=timezone.now)
    description = models.CharField(max_length=50, blank=True, null=True)
    invoicce_number = models.CharField(max_length=50, blank=True, null=True, default='N/A')
    inward = models.IntegerField(default=0)
    outward = models.IntegerField(default=0) 
    balance = models.IntegerField(default=0) 
    productprice = models.ForeignKey(ProductPrice, on_delete=models.PROTECT, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    
    
    class Meta:
        verbose_name_plural = "Stocks"
    
    def __str__(self):
        return f'{self.productprice.product} {self.inward}  {self.outward} {self.balance}'
    
    
    
class StockRequisitionReturn(models.Model):
    invoicce_number = models.CharField(max_length=50, blank=True, null=True, default='N/A')
    balance = models.IntegerField(default=0) 
    productprice = models.ForeignKey(ProductPrice, on_delete=models.PROTECT, null=True, blank=True)
    srv_no = models.CharField(max_length=20)
    supplied_to = models.CharField(blank=True, null=True, max_length=50)
    on_account_of = models.CharField(blank=True, null=True, max_length=50)
    qty = models.IntegerField(default=0)  
    product_owner = models.CharField(max_length=50, blank=True, null=True)
    recipient = models.CharField(max_length=50, blank=True, null=True)
    cashier = models.CharField(max_length=50, blank=True, null=True)
    stock_keeper = models.CharField(max_length=50, blank=True, null=True)
    date = models.DateField(default=timezone.now)
    
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    date_created = models.DateTimeField(default=timezone.now)
    
   
    class Meta:
        verbose_name_plural = "Stock Requisition"
    
    def __str__(self):
        return f'{self.productprice.product} {self.recipient}  {self.balance}'
   
# comments for initial credit sale approval
class approval_comment(models.Model):
    comments = models.CharField(max_length=256)
    date_created = models.DateTimeField(default=timezone.now)
    date_edited = models.DateTimeField(auto_now=timezone.now) 
    
    class Meta:
        verbose_name_plural = "Approval Comments"
        
    def __str__(self):
        return f'{self.comments}'
    
    
    
class CreditSaleApproval(models.Model):
    status_choices = (('Pending', 'Pending'), ('Approve', 'Approve'), ('Reject', 'Reject'))
    sales_date = models.DateTimeField(default=timezone.now)
    sales_id = models.CharField(max_length=50, null=True, blank=True, default='PD-123')
    customer = models.CharField(max_length=64, blank=True, null=True)
    credit_customer = models.ForeignKey(Customer, on_delete=models.PROTECT, null=True, blank=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2)
    selling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_discount = models.DecimalField(max_digits=10, decimal_places=2)
    company_location = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.CharField(max_length=64, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    momo = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cash = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    visa = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set')
    payee_name = models.CharField(max_length=64, null=True, blank=True)
    discount_authorizer = models.CharField(max_length=64, null=True, blank=True)
    credit_authorizer = models.CharField(max_length=64, null=True, blank=True)
    sale_type = models.CharField(max_length=64, null=True, blank=True)
    status = models.CharField(max_length=64, null=True, blank=True, choices=status_choices, default='Pending')
    approved_rejected_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='approved_%(class)s_set')
    comments = models.ForeignKey(approval_comment, on_delete=models.CASCADE, null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Credit Sale Approval"
        
          
    def __str__(self):
        return f'{self.customer} {self.net_amount} {self.sales_id}'
    
    
    
class CreditSaleApprovalItems(models.Model):
    sales = models.ForeignKey(CreditSaleApproval, on_delete=models.CASCADE)
    product = models.CharField(max_length=50)
    package = models.CharField(max_length=50)
    total = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2)
    qty = models.CharField(max_length=50, null=True, blank=True)
    unit_discount = models.DecimalField(max_digits=10, decimal_places=2)
    productprice_id = models.IntegerField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)    
    
    
    class Meta:
        verbose_name_plural = "Credit Sale Approval Items"
        
        
    def __str__(self):
        return f'{self.product} {self.total}'
    
    
    
    
class CustomerStatementReport(models.Model):
    transaction_date = models.DateTimeField(default=timezone.now)
    customer = models.ForeignKey(Customer, on_delete=models.PROTECT)
    transaction_number = models.CharField(max_length=10, default="", blank=True, null=True)
    description = models.CharField(max_length=100)
    debit = models.DecimalField(decimal_places=2, max_digits=12)
    credit = models.DecimalField(decimal_places=2, max_digits=12)
    balance = models.DecimalField(decimal_places=2, max_digits=12)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    def __str__(self):
        return 
    
    
    
    
class UserActivitiesReport(models.Model):
    user = models.ForeignKey(User, on_delete=models.PROTECT)
    activity = models.CharField(max_length=250)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
    
    
    def __str__(self):
        return f'{self.user} {self.activity}'
    
    
    
    
    
class ReturnedSales(models.Model):
    sales_date = models.DateTimeField(default=timezone.now)
    sales_id = models.CharField(max_length=50, null=True, blank=True, default='PD-123')
    customer = models.CharField(max_length=64, blank=True, null=True)
    credit_customer = models.ForeignKey(Customer, on_delete=models.PROTECT, null=True, blank=True)
    total_amount = models.DecimalField(max_digits=10, decimal_places=2)
    selling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    total_discount = models.DecimalField(max_digits=10, decimal_places=2)
    company_location = models.ForeignKey(CompanyLocation, on_delete=models.PROTECT, null=True)
    net_amount = models.DecimalField(max_digits=10, decimal_places=2)
    paid_amount = models.DecimalField(max_digits=10, decimal_places=2)
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque_number = models.CharField(max_length=50, null=True, blank=True)
    cheque_date = models.DateTimeField(null=True, blank=True)
    bank = models.CharField(max_length=64, null=True, blank=True)
    bank_branch = models.CharField(max_length=50, null=True, blank=True)
    momo = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cash = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    cheque = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    visa = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='CreatedBy')
    returned_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='ReturnedSalesBy')
    payee_name = models.CharField(max_length=64, null=True, blank=True)
    discount_authorizer = models.CharField(max_length=64, null=True, blank=True)
    sale_type = models.CharField(max_length=64, null=True, blank=True)
    status = models.BooleanField(default=False)
    status_approved_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='ApprovedBy')
    stock_approval_date = models.DateTimeField(null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)
        
    class Meta:
        verbose_name_plural = "Sales"
        
          
    def __str__(self):
        return f'{self.customer} {self.net_amount} {self.sales_id}'
    
    
    
    
class ReturnedSalesItem(models.Model):
    sales = models.ForeignKey(Sales, on_delete=models.CASCADE)
    product = models.CharField(max_length=50)
    package = models.CharField(max_length=50)
    total = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=10, decimal_places=2)
    qty = models.CharField(max_length=50, null=True, blank=True)
    unit_discount = models.DecimalField(max_digits=10, decimal_places=2)
    productprice_id = models.IntegerField(null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True, related_name='created_%(class)s_set') 
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now)    
    
    
    class Meta:
        verbose_name_plural = "Sale Items"
        
    
        
    def __str__(self):
        return f'{self.product} {self.total - (self.unit_discount*int(self.qty)) }'
    
    
    
    


class CustomerData(models.Model):
    name = models.CharField(max_length=50)
    phone = models.CharField(max_length=50)
    date_created = models.DateTimeField(default=timezone.now)
    date_updated = models.DateTimeField(auto_now=timezone.now) 
    
    
    class Meta:
        verbose_name_plural = "Customer Database"
      
    def __str__(self):
        return self.name + ' '+ self.phone  
    
    
    